import { ModelDefinition } from "./model_definitions";
import { ObjectID, FilterQuery } from "mongodb";
import SafeMongo from ".";
export interface ModelDataBase {
    _id: ObjectID;
    _v: number;
}
declare type FilterProperties<T, U> = {
    [key in keyof T]: T[key] extends U ? key : never;
};
declare type AllowedNames<T, U> = FilterProperties<T, U>[keyof T];
export declare class InvalidObjectError extends Error {
    constructor();
}
export default class Model<T extends ModelDataBase> {
    private _sm;
    private _definition;
    private _collection;
    private _version;
    private _new;
    private _fetched;
    readonly version: number;
    constructor(_sm: SafeMongo, _definition: ModelDefinition);
    new(data?: Partial<T>): T;
    validate(data: T): void;
    save(data: T): Promise<void>;
    findById(id: ObjectID | string): Promise<T | null>;
    findOne(filter: FilterQuery<T>): Promise<T | null>;
    find(filter: FilterQuery<T>): Promise<T[]>;
    delete(doc: T | ObjectID | string): Promise<void>;
    deleteFilter(filter: FilterQuery<T>): Promise<void>;
    incement(elm: ObjectID | string | T, field: AllowedNames<T, number>, amount?: number): Promise<void>;
    private _add_fetched;
    private _upgrade;
    private _validate;
}
export declare function recursiveDeepCopy<T = any | any[]>(o: T): T;
export {};
